<?
/**
* Library for HTML Forms
*
* SmartForm is a library to create HTML Forms with arbitrary validation
* attached to each form element, all treated on the same block of code
*
* @author	Sergio D. Bruder <bruder@conectiva.com.br> and Paulo Ivo <fred@conectiva.com.br>
* @version	0.1
*
*/


/**
* HTML Form start code
*
* function that outputs the HTML code for form start. First it searchs for
* an element named 'FORM_START' in the wildweb template system. If wildweb
* is not present or FORM_START element doesnt exists, a function named 
* FORM_START() is searched for. If the function doesnt exists then it
* outputs a hardcoded HTML block.
*
* @access	private
* @group	html_generation
* @param	string	$title		Title to be used for the form
* @param	string	$comment	little text (can be used as coment, instructions, etc)
* @return	Nothing
*/
function sfTemplateStart($title,$comment) {
	if (function_exists('sty_exec_element')) {
		if (!sty_exec_element('FORM_START')) {
			$default = true;
		}
	} else {
		if (function_exists('FORM_START')) {
			FORM_START($title,$comment);
		} else {
			$default = true;
		}
	}
	if ($default) {
		echo "<TABLE BORDER=0 CELLPADDING=2 CELLSPACING=0>\n";
		echo "<TR><TD COLSPAN=3><strong><font size=+1>$title</font></strong><BR>";
		echo "<font size=-1>$comment</font></TD></TR>\n";
	}
}


/**
* HTML Form field code 
*
* outputs HTML code responsible to hold title, the field itself and a comment.
* we use the same 'tries' of sfTemplateStart() to get that value, ie, wildweb 
* template system first, the a function named 'FORM_FIELD', then a standard HTML
* block is used.
*
* @access	private
* @group	html_generation
* @param	array	$FH		form handler
* @param	string	$title		Title of the current field
* @param	string	$field		The field itself
* @param	string	$comment	little text (can be used as coment, instructions, etc)
* @return	Nothing
*/
function sfTemplate($FH,$title,$field,$comment='') {
	if ($FH['__VALID']) {
		if (function_exists('sty_exec_element')) {
			if (!sty_exec_element('FORM_FIELD')) {
				$default = true;
			};
		} else {
			if (function_exists('FORM_FIELD')) {
				FORM_FIELD($title,$field,$comment);
			} else {
				$default = true;
			}
		}
		if ($default) {
			echo "<TR><TD ALIGN=\"right\" NOWRAP><strong>&nbsp;$title</strong></TD>";
			echo "<TD VALIGN=\"center\">$field</TD>";
			echo "<TD><font size=-1>&nbsp;$comment</font></TD></TR>\n";
		}
	} else {
		sfTemplateError($title,$field,$FH['__COMMENT']);
	}
}


/**
* HTML Form invalid field code 
*
* outputs HTML code responsible to hold title, the field itself and a comment
* for invalid fields.  we use the same 'tries' of sfTemplateStart() to get that
* value, ie, wildweb template system first, the a function named 'FORM_FIELD',
* then a standard HTML block is used.
*
* @access	private
* @group	html_generation
* @param	string	$title		Title of the current field
* @param	string	$field		The field itself
* @param	string	$comment	little text (can be used as coment, instructions, etc)
* @return	Nothing
*/
function sfTemplateError($title,$field,$comment='') {
	if (function_exists('sty_exec_element')) {
		if (!sty_exec_element('FORM_FIELDERR')) {
			$default = true;
		};
	} else {
		if (function_exists('FORM_FIELDERR')) {
			FORM_FIELD($title,$field,$comment);
		} else {
			$default = true;
		}
	}
	if ($default) {
		echo "<TR bgcolor=\"#ff8000\"><TD ALIGN=\"right\" NOWRAP><strong>&nbsp;$title</strong></TD>";
		echo "<TD VALIGN=\"center\">$field</TD>";
		echo "<TD><font size=-1>&nbsp;$comment</font></TD></TR>\n";
	}
}


/**
* outputs non-viewable fields of the form 
*
* actually it only outputs the non-viewable fields. Its a placeholder for
* future needs. 
*
* @access	private
* @group	html_generation
* @param	string	$field		The field itself
* @return	Nothing
*/
function sfNoTemplate($field) {
	echo $field;
}


/**
* HTML Form end code
*
* function that outputs the HTML code for form end. First it searchs for
* an element named 'FORM_END' in the wildweb template system. If wildweb
* is not present or FORM_END element doesnt exists, a function named 
* FORM_END() is searched for. If the function doesnt exists then it
* outputs a hardcoded HTML block.
*
* @access	private
* @group	html_generation
* @param	string	$footnote	a footnote text (can be used as coment, instructions, etc)
* @return	Nothing
*/
function sfTemplateEnd($footnote='') {
	if (function_exists('sty_exec_element')) {
		if (!sty_exec_element('FORM_END')) {
			$default = true;
		}
	} else {
		if (function_exists('FORM_END')) {
			FORM_END($footnote);
		} else {
			$default = true;
		}
	}
	if ($default) {
		if ($footnote) {
			echo "<TR><TD colspan=3><font size=-1>$footnote</font></TD></TR>\n";
		}
		echo "</TABLE>\n";
	}
}


/**
* Creates the 'FORM HANDLER' 
*
* creates the FORM_HANDLER used in all other functions to attach additional
* fields and validators. Some initial parameters are set, as the name of the
* PHP variable that will hold the field values, a title for the form, the
* method used to transmit the form, etcetera. 
*
* @access	public
* @group	public_interface
* @param	array	$FH		form handler
* @param	string	$formname	actual name of the PHP variable that will hold the form values, usually form_handler itself.
* @param	string	$title		title for the form
* @param	string	$comment	text comment usually some small text bellow title
* @param	string	$action		the URL that will receive the execution of this form, defaults to $PHP_SELF
* @param	string	$method		the used method to pass variables , 'GET' or 'POST', defaults to 'POST'
* @return	form_handler
*/
function sfOpenForm($FH,$formname='form',$title='SmartForm Test',$comment='',$action='',$method='POST') {
	global $PHP_SELF;
	$action = ($action=='') ? $PHP_SELF : $action;
	if ($FH['__OPTION']=='SHOW') {
		echo "<FORM ACTION=\"$action\" METHOD=\"$method\">\n";
		echo "<INPUT TYPE=\"hidden\" NAME=\"__SFEXEC\" VALUE=\"EXEC\">\n";
		sfTemplateStart($title,$comment);
	} else {
		$FH[]=array('formstart',$formname,$title,$comment,$action,$method);
		return $FH;
	}
}

function sfCloseForm($FH,$footnote='') {
	global $PHP_SELF;
	if ($FH['__OPTION']=='SHOW') {
		sfTemplateEnd($footnote);
		echo "</FORM>\n";
	} else {
		$FH[]=array('formend',$footnote);
		return $FH;
	}
}

function sfButtons($FH,$buttons) {
	if ($FH['__OPTION']=='SHOW') {
		$btns = '';
		reset($buttons);
		while ($el = each($buttons)) {
			$btn = explode("|",$el[1]);
			if ($btn[0]=="RESET") {
				$btns .= '<INPUT TYPE=reset VALUE="'.$btn[1].'">&nbsp;';
			} else {
				$btns .= '<INPUT TYPE=submit NAME="'.$FH[0][1].'[EXEC]" '.
					'VALUE="'.$btn[1].'">&nbsp;';
			}
		}
		sfTemplate($FH, '', $btns);
	} else {
		$FH[]=array('button',$buttons,'VALID'=>1);
		return $FH;
	}
}


function sfCorrectValue($fh,$name,$value) {
	global $__SFEXEC;
	if ($__SFEXEC == 'EXEC') {
		return $GLOBALS[$fh[0][1]][$name];
	} else {
		return $value;
	}
}

function sfText($FH,$title,$name,$value='',$size=40,$maxl=40,$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		sfTemplate($FH,$title, "<INPUT NAME=\"".$FH[0][1]."[$name]\" SIZE=$size MAXLENGTH=$maxl VALUE=\"$value\">",$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('text',$title,$name,$value,$size,$maxl,$comment,'NAME'=>$name,'VALID'=>true);
		return $FH;
	}
}

function sfPassword($FH,$title,$name,$value='',$size=20,$maxl=20,$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		sfTemplate($FH, $title, "<INPUT NAME=\"".$FH[0][1]."[$name\" TYPE=\"password\" SIZE=$size MAXLENGTH=$maxl VALUE=\"$value\">",$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('password',$title,$name,$value,$size,$maxl,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfTextArea($FH,$title,$name,$value='',$cols=50,$rows=10,$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		sfTemplate($FH, $title, 
			"<TEXTAREA NAME=\"".$FH[0][1]."[$name]\" COLS=$cols ROWS=$rows WRAP=\"virtual\">$value</TEXTAREA>",
			$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('textarea',$title,$name,$value,$cols,$rows,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfTextAreaNoWrap($FH,$title,$name,$value='',$cols=50,$rows=10,$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		sfTemplate($FH, $title, 
			"<TEXTAREA NAME=\"".$FH[0][1]."[$name]\" COLS=$cols ROWS=$rows>$value</TEXTAREA>",
			$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('textareanowrap',$title,$name,$value,$cols,$rows,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfHidden($FH,$name,$value='') {
	if ($FH['__OPTION']=='SHOW') {
		sfNoTemplate("<INPUT TYPE=\"hidden\" NAME=\"".$FH[0][1]."[$name]\" VALUE=\"$value\">");
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('hidden',$name,$value,'NAME'=>$name);
		return $FH;
	}
}

function sfRadio($FH,$title,$name,$value='radio',$label='Label',$checked='',$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		$chk = ($FH[$name]==$value) ? " CHECKED" : "";
		sfTemplate($FH, $title,"<INPUT TIPE=\"radio\" NAME=\"".$FH[0][1]."[$name]\" VALUE=\"$value\"$chk>$label", $comment); 
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('radio',$title,$name,$value,$label,$checked,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfDBLookup($FH,$title,$name,$value='',$lktable,$lkid,$lkdescr,$comment='') {
	global $sql;
	if ($FH['__OPTION']=='SHOW') {
		$field = "<SELECT NAME=\"".$FH[0][1]."[$name]"."[]"."\" SINGLE>";
		$qry = db_query($sql,"SELECT $lkid, $lkdescr FROM $lktable GROUP BY $lkdescr");
		while ($lk = fetch_array($qry)) {
			$field .= "<OPTION VALUE=" . $lk[$lkid];
			if ($lk[$lkid] == $value) {
				$field .= "SELECTED";
			}
			$field .= ">".$lk[$lkdescr]."</OPTION>";
		}
		$field .= "</SELECT>";
		sfTemplate($FH, $title,$field,$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('dblookup',$title,$name,$value,$lktable,$lkid,$lkdescr,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfMultDBLookup($FH,$title,$name,$value='',$size=10,$lktable,$lkid,$lkdescr,$comment='') {
	global $sql;
	if ($FH['__OPTION']=='SHOW') {
		$field = "<SELECT NAME=\"".$FH[0][1]."[$name]"."[]"."\" MULTIPLE SIZE=\"$size\">";
		$qry = db_query($conexao,"SELECT $lkid, $lkdescr FROM $lktable GROUP BY $lkdescr");
		while ($lk = fetch_array($qry)) {
			$field .= "<OPTION VALUE=" . $lk[$lkid];
			if ($lk[$lkid] == $value) {
				$field .= "SELECTED";
			}
			$field .= ">".$lk[$lkdescr]."</OPTION>";
		}
		$field .= "</SELECT>";
		sfTemplate($FH, $title,$field,$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('multdblookup',$title,$name,$value,$size,$lktable,$lkid,$lkdescr,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfLookup($FH,$title,$name,$value='',$values='',$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		$field = "<SELECT NAME=\"".$FH[0][1]."[$name]"."[]"."\" SINGLE>";
		while ($lk = each($values)) {
			$field .= "<OPTION VALUE=" . $lk[0];
			if ($lk[0] == $value) {
				$field .= "SELECTED";
			}
			$field .= ">".$lk[1]."</OPTION>";
		}
		$field .= "</SELECT>";
		sfTemplate($FH, $title,$field,$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('lookup',$title,$name,$value,$values,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sfMultLookup($FH,$title,$name,$value='',$size=10,$values='',$comment='') {
	if ($FH['__OPTION']=='SHOW') {
		$field = "<SELECT NAME=\"".$FH[0][1]."[$name]"."[]"."\" MULTIPLE SIZE=\"$size\">";
		while ($lk = each($values)) {
			$field .= "<OPTION VALUE=" . $lk[0];
			if ($lk[0] == $value) {
				$field .= "SELECTED";
			}
			$field .= ">".$lk[1]."</OPTION>";
		}
		$field .= "</SELECT>";
		sfTemplate($FH, $title,$field,$comment);
	} else {
		$value = sfCorrectValue($FH,$name,$value);
		$FH[]=array('multlookup',$title,$name,$value,$size,$values,$comment,'NAME'=>$name);
		return $FH;
	}
}

function sf_show($fh) {
	$fh['__OPTION']='SHOW';
	$fh['__VALID']   = 1;
	$fh['__COMMENT'] = '';
	reset($fh);
	while ( list( $duh, $el ) = each($fh) ) {
		$fh['__VALID']   = $el['VALID'];
		$fh['__COMMENT'] = $el['COMMENT'];
		switch ($el[0]) {
		case 'formstart':
			sfOpenForm($fh,$el[1],$el[2],$el[3],$el[4],$el[5]);
			break;
		case 'formend':
			sfCloseForm($fh,$el[1]);
			break;
		case 'button':
			sfButtons($fh,$el[1]);
			break;
		case 'text':
			sfText($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		case 'password':
			sfPassword($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		case 'textarea':
			sfTextArea($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		case 'textareanowrap':
			sfTextAreaNoWrap($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		case 'hidden':
			sfHidden($fh,$el[1],$el[2]);
			break;
		case 'radio':
			sfRadio($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		case 'dblookup':
			sfDBLookup($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6],$el[7]);
			break;
		case 'multdblookup':
			sfMultDBLookup($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6],$el[7],$el[8]);
			break;
		case 'lookup':
			sfLookup($fh,$el[1],$el[2],$el[3],$el[4],$el[5]);
			break;
		case 'multlookup':
			sfMultLookup($fh,$el[1],$el[2],$el[3],$el[4],$el[5],$el[6]);
			break;
		}
	}
	unset($fh['__OPTION']);
	unset($fh['__VALID']);
	unset($fh['__COMMENT']);
	return '';
}


function sfValidRegExp($fh,$regexp,$message='',$value='') {
	if ($fh['__OPTION']=='VALIDATE') {
		return ereg($regexp,$value);
	} else {
		$el = array_pop($fh);
		$el['VALIDATORS'][] = array('regexp',$regexp,$message);
		array_push($fh,$el);
		return $fh;
	}
}


function sfValidIRegExp($fh,$regexp,$message='',$value='') {
	if ($fh['__OPTION']=='VALIDATE') {
		return eregi($regexp,$value);
	} else {
		$el = array_pop($fh);
		$el['VALIDATORS'][] = array('iregexp',$regexp,$message);
		array_push($fh,$el);
		return $fh;
	}
}


function sfValidNotEmpty($fh,$message='',$value='') {
	return sfValidRegExp($fh,'[^ ]+',$message,$value);
}


function sf_validfield($fh,&$el,$value) {
	$ValidField = true;
	if (isset($el['VALIDATORS'])) {
		reset($el['VALIDATORS']);
		while ((list($garbage,$vl) = each($el['VALIDATORS'])) and $ValidField ) {
			switch ($vl[0]) {
			case 'regexp':
				$valid = sfValidRegExp($fh,$vl[1],$vl[2],$value);
				$comment = $vl[2];
				break;
			case 'iregexp':
				$valid = sfValidIRegExp($fh,$vl[1],$vl[2],$value);
				$comment = $vl[2];
				break;
			}
			$ValidField = (($valid>0) and $ValidField);
			if ($valid == 0) {
				$el['VALID']=0;
				$el['COMMENT']=$comment;
			} else {
				$el['VALID']=1;
				$el['COMMENT']='';
			}
		}
	}
	return $ValidField;
}


function sf_validate($fh) {
	$fh['__OPTION']='VALIDATE';
	$fname = $fh[0][1];
	reset($fh);
	$ValidForm = true;
	while (list($index,$el) = each($fh)) {
		if ($el['NAME'] and gettype($index)=='integer' and gettype($el)=='array') {
			$value = $GLOBALS[$fname][$el['NAME']];
			$fh[$index]['VALUE'] = $value;
			$valid = sf_validfield($fh,$el,$value);
			$fh[$index]['VALID'] = $valid; 
			$fh[$index]['COMMENT'] = $el['COMMENT']; 
			$ValidForm = ($ValidForm and ($valid>0));
		}
	}
	$fh['__VALIDFORM'] = $ValidForm;
	unset($fh['__OPTION']);
	echo $Validform;
	if ($ValidForm) {
		reset($fh);
		while (list($index,$value) = each($fh)) {
			if (strlen(ereg_replace('[0-9]*','',$index))==0) {
				unset($fh[$index]);
			}
		}
		return $fh;
	} else {
		sf_show($fh);
	}
}


function sf_exec($fh) {
	global $__SFEXEC;
	if ($__SFEXEC) {
		return sf_validate($fh);
	} else {
		return sf_show($fh);
	}
}


function sf_Valid(&$fh) {
	$validform = $fh['__VALIDFORM'];
	unset($fh['__VALIDFORM']);
	return $validform;
}


?>
